@@ -1,9 +1,19 @@
 /*
- * Copyright (c) 2010, 2011 Nicira Networks.
- * Distributed under the terms of the GNU GPL version 2.
+ * Copyright (c) 2007-2012 Nicira Networks.
  *
- * Significant portions of this file may be copied from parts of the Linux
- * kernel, by Linus Torvalds and others.
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of version 2 of the GNU General Public
+ * License as published by the Free Software Foundation.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+ * 02110-1301, USA
  */
 
 #define pr_fmt(fmt) KBUILD_MODNAME ": " fmt
@@ -25,10 +35,10 @@
 #include "vport-netdev.h"
 
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,37) && \
-    !defined(HAVE_VLAN_BUG_WORKAROUND)
+	!defined(HAVE_VLAN_BUG_WORKAROUND)
 #include <linux/module.h>
 
-static int vlan_tso __read_mostly = 0;
+static int vlan_tso __read_mostly;
 module_param(vlan_tso, int, 0644);
 MODULE_PARM_DESC(vlan_tso, "Enable TSO for VLAN packets");
 #else
@@ -47,7 +57,7 @@ static rx_handler_result_t netdev_frame_hook(struct sk_buff **pskb)
 	if (unlikely(skb->pkt_type == PACKET_LOOPBACK))
 		return RX_HANDLER_PASS;
 
-	vport = netdev_get_vport(skb->dev);
+	vport = ovs_netdev_get_vport(skb->dev);
 
 	netdev_port_receive(vport, skb);
 
@@ -62,7 +72,7 @@ static struct sk_buff *netdev_frame_hook(struct sk_buff *skb)
 	if (unlikely(skb->pkt_type == PACKET_LOOPBACK))
 		return skb;
 
-	vport = netdev_get_vport(skb->dev);
+	vport = ovs_netdev_get_vport(skb->dev);
 
 	netdev_port_receive(vport, skb);
 
@@ -120,7 +130,8 @@ static struct vport *netdev_create(const struct vport_parms *parms)
 	struct netdev_vport *netdev_vport;
 	int err;
 
-	vport = vport_alloc(sizeof(struct netdev_vport), &netdev_vport_ops, parms);
+	vport = ovs_vport_alloc(sizeof(struct netdev_vport),
+				&ovs_netdev_vport_ops, parms);
 	if (IS_ERR(vport)) {
 		err = PTR_ERR(vport);
 		goto error;
@@ -136,7 +147,7 @@ static struct vport *netdev_create(const struct vport_parms *parms)
 
 	if (netdev_vport->dev->flags & IFF_LOOPBACK ||
 	    netdev_vport->dev->type != ARPHRD_ETHER ||
-	    is_internal_dev(netdev_vport->dev)) {
+	    ovs_is_internal_dev(netdev_vport->dev)) {
 		err = -EINVAL;
 		goto error_put;
 	}
@@ -157,7 +168,7 @@ static struct vport *netdev_create(const struct vport_parms *parms)
 error_put:
 	dev_put(netdev_vport->dev);
 error_free_vport:
-	vport_free(vport);
+	ovs_vport_free(vport);
 error:
 	return ERR_PTR(err);
 }
@@ -173,10 +184,10 @@ static void netdev_destroy(struct vport *vport)
 	synchronize_rcu();
 
 	dev_put(netdev_vport->dev);
-	vport_free(vport);
+	ovs_vport_free(vport);
 }
 
-int netdev_set_addr(struct vport *vport, const unsigned char *addr)
+int ovs_netdev_set_addr(struct vport *vport, const unsigned char *addr)
 {
 	struct netdev_vport *netdev_vport = netdev_vport_priv(vport);
 	struct sockaddr sa;
@@ -187,49 +198,49 @@ int netdev_set_addr(struct vport *vport, const unsigned char *addr)
 	return dev_set_mac_address(netdev_vport->dev, &sa);
 }
 
-const char *netdev_get_name(const struct vport *vport)
+const char *ovs_netdev_get_name(const struct vport *vport)
 {
 	const struct netdev_vport *netdev_vport = netdev_vport_priv(vport);
 	return netdev_vport->dev->name;
 }
 
-const unsigned char *netdev_get_addr(const struct vport *vport)
+const unsigned char *ovs_netdev_get_addr(const struct vport *vport)
 {
 	const struct netdev_vport *netdev_vport = netdev_vport_priv(vport);
 	return netdev_vport->dev->dev_addr;
 }
 
-struct kobject *netdev_get_kobj(const struct vport *vport)
+struct kobject *ovs_netdev_get_kobj(const struct vport *vport)
 {
 	const struct netdev_vport *netdev_vport = netdev_vport_priv(vport);
 	return &netdev_vport->dev->NETDEV_DEV_MEMBER.kobj;
 }
 
-unsigned netdev_get_dev_flags(const struct vport *vport)
+unsigned ovs_netdev_get_dev_flags(const struct vport *vport)
 {
 	const struct netdev_vport *netdev_vport = netdev_vport_priv(vport);
 	return dev_get_flags(netdev_vport->dev);
 }
 
-int netdev_is_running(const struct vport *vport)
+int ovs_netdev_is_running(const struct vport *vport)
 {
 	const struct netdev_vport *netdev_vport = netdev_vport_priv(vport);
 	return netif_running(netdev_vport->dev);
 }
 
-unsigned char netdev_get_operstate(const struct vport *vport)
+unsigned char ovs_netdev_get_operstate(const struct vport *vport)
 {
 	const struct netdev_vport *netdev_vport = netdev_vport_priv(vport);
 	return netdev_vport->dev->operstate;
 }
 
-int netdev_get_ifindex(const struct vport *vport)
+int ovs_netdev_get_ifindex(const struct vport *vport)
 {
 	const struct netdev_vport *netdev_vport = netdev_vport_priv(vport);
 	return netdev_vport->dev->ifindex;
 }
 
-int netdev_get_mtu(const struct vport *vport)
+int ovs_netdev_get_mtu(const struct vport *vport)
 {
 	const struct netdev_vport *netdev_vport = netdev_vport_priv(vport);
 	return netdev_vport->dev->mtu;
@@ -238,10 +249,11 @@ int netdev_get_mtu(const struct vport *vport)
 /* Must be called with rcu_read_lock. */
 static void netdev_port_receive(struct vport *vport, struct sk_buff *skb)
 {
-	if (unlikely(!vport)) {
-		kfree_skb(skb);
-		return;
-	}
+	if (unlikely(!vport))
+		goto error;
+
+	if (unlikely(skb_warn_if_lro(skb)))
+		goto error;
 
 	/* Make our own copy of the packet.  Otherwise we will mangle the
 	 * packet for anyone who came before us (e.g. tcpdump via AF_PACKET).
@@ -253,16 +265,19 @@ static void netdev_port_receive(struct vport *vport, struct sk_buff *skb)
 
 	skb_push(skb, ETH_HLEN);
 
-	if (unlikely(compute_ip_summed(skb, false))) {
-		kfree_skb(skb);
-		return;
-	}
+	if (unlikely(compute_ip_summed(skb, false)))
+		goto error;
+
 	vlan_copy_skb_tci(skb);
 
-	vport_receive(vport, skb);
+	ovs_vport_receive(vport, skb);
+	return;
+
+error:
+	kfree_skb(skb);
 }
 
-static inline unsigned packet_length(const struct sk_buff *skb)
+static unsigned packet_length(const struct sk_buff *skb)
 {
 	unsigned length = skb->len - ETH_HLEN;
 
@@ -294,13 +309,10 @@ static int netdev_send(struct vport *vport, struct sk_buff *skb)
 	if (unlikely(packet_length(skb) > mtu && !skb_is_gso(skb))) {
 		if (net_ratelimit())
 			pr_warn("%s: dropped over-mtu packet: %d > %d\n",
-				dp_name(vport->dp), packet_length(skb), mtu);
+				ovs_dp_name(vport->dp), packet_length(skb), mtu);
 		goto error;
 	}
 
-	if (unlikely(skb_warn_if_lro(skb)))
-		goto error;
-
 	skb->dev = netdev_vport->dev;
 	forward_ip_summed(skb, true);
 
@@ -367,18 +379,18 @@ tag:
 
 error:
 	kfree_skb(skb);
-	vport_record_error(vport, VPORT_E_TX_DROPPED);
+	ovs_vport_record_error(vport, VPORT_E_TX_DROPPED);
 	return 0;
 }
 
 /* Returns null if this device is not attached to a datapath. */
-struct vport *netdev_get_vport(struct net_device *dev)
+struct vport *ovs_netdev_get_vport(struct net_device *dev)
 {
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,36)
 #if IFF_BRIDGE_PORT != IFF_OVS_DATAPATH
 	if (likely(dev->priv_flags & IFF_OVS_DATAPATH))
 #else
-	if (likely(rcu_access_pointer(dev->rx_handler) == netdev_frame_hook))	
+	if (likely(rcu_access_pointer(dev->rx_handler) == netdev_frame_hook))
 #endif
 		return (struct vport *)rcu_dereference_rtnl(dev->rx_handler_data);
 	else
@@ -388,40 +400,31 @@ struct vport *netdev_get_vport(struct net_device *dev)
 #endif
 }
 
-const struct vport_ops netdev_vport_ops = {
+const struct vport_ops ovs_netdev_vport_ops = {
 	.type		= OVS_VPORT_TYPE_NETDEV,
 	.flags          = VPORT_F_REQUIRED,
 	.init		= netdev_init,
 	.exit		= netdev_exit,
 	.create		= netdev_create,
 	.destroy	= netdev_destroy,
-	.set_addr	= netdev_set_addr,
-	.get_name	= netdev_get_name,
-	.get_addr	= netdev_get_addr,
-	.get_kobj	= netdev_get_kobj,
-	.get_dev_flags	= netdev_get_dev_flags,
-	.is_running	= netdev_is_running,
-	.get_operstate	= netdev_get_operstate,
-	.get_ifindex	= netdev_get_ifindex,
-	.get_mtu	= netdev_get_mtu,
+	.set_addr	= ovs_netdev_set_addr,
+	.get_name	= ovs_netdev_get_name,
+	.get_addr	= ovs_netdev_get_addr,
+	.get_kobj	= ovs_netdev_get_kobj,
+	.get_dev_flags	= ovs_netdev_get_dev_flags,
+	.is_running	= ovs_netdev_is_running,
+	.get_operstate	= ovs_netdev_get_operstate,
+	.get_ifindex	= ovs_netdev_get_ifindex,
+	.get_mtu	= ovs_netdev_get_mtu,
 	.send		= netdev_send,
 };
 
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,36)
 /*
- * In kernels earlier than 2.6.36, Open vSwitch cannot safely coexist with
- * the Linux bridge module on any released version of Linux, because there
- * is only a single bridge hook function and only a single br_port member
- * in struct net_device.
- *
- * Declaring and exporting this symbol enforces mutual exclusion.  The bridge
- * module also exports the same symbol, so the module loader will refuse to
- * load both modules at the same time (e.g. "bridge: exports duplicate symbol
- * br_should_route_hook (owned by openvswitch_mod)").
- *
- * The use of "typeof" here avoids the need to track changes in the type of
- * br_should_route_hook over various kernel versions.
+ * In kernels earlier than 2.6.36, Open vSwitch cannot safely coexist with the
+ * Linux bridge module, because there is only a single bridge hook function and
+ * only a single br_port member in struct net_device, so this prevents loading
+ * both bridge and openvswitch_mod at the same time.
  */
-typeof(br_should_route_hook) br_should_route_hook;
-EXPORT_SYMBOL(br_should_route_hook);
+BRIDGE_MUTUAL_EXCLUSION;
 #endif
